//~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~
//
// Archivo: Heap.h
//
// Propsito:
//    Define las clases necesarias para dar una definicin orientada a objetos del sistema de
//    gestin de montones del API Win32.
//
//    Las clases definidas son:
//        CHeapBlock:     representa un bloque reservado con HeapAlloc.
//        CHeap:          representa un montn creado con HeapCreate. Se compone de uno o 
//                        varios bloques de tipo CHeapBlock.
//        CHeapList:      representa una lista de punteros almacenada en un montn dedicado.
//        CHeapBlockList: especializacin de CHeapList que maneja objetos de tipo CHeapBlock
//                        en vez de punteros genricos.
//
// Autor:          Jos Manuel Navarro (jose.man_ARROBA_iespana.es)
//
// Fecha:          01/11/2002: Creacin
//                 15/11/2002: Se incluyen los smbolos IN, OUT y OPTIONAL en los parmetros
//                             Se cambia los tipos de datos int por INT y void por VOID
//
// Observaciones:  Clases creadas en Visual C++ 6 para el artculo "Los montones en Win32"
//                 publicado en http://users.servicios.retecal.es/sapivi/prog/cpp/montones.html
//
// Copyright:      Este cdigo es de dominio pblico y se puede utilizar y/o mejorar siempre 
//                 que SE HAGA REFERENCIA AL AUTOR ORIGINAL, ya sea a travs de estos 
//                 comentarios o de cualquier otro modo.
//
//~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~

#include <windows.h>

//
// smbolo condicional HEAP_EXPORTS indica que las funciones se exportarn.
// Este smbolo se define en "Heap.cpp", as que slo se exportarn las funciones cuando se
// incluya su implementacin. En caso contrario, se importarn.
//
#ifdef HEAP_EXPORTS
  #define    HEAP_API    __declspec(dllexport)
#else
  #define    HEAP_API    __declspec(dllimport)
#endif


#define      INITIAL_SIZE    1024 * 64
#define      MAX_LIST_SIZE   134217727


class CHeap;

//~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~
//
// Clase: CHeapBlock
//
// Descripcin: representa un bloque de memoria reservada dentro de un montn.
//
// Uso: Se puede crear este objeto asociado a un montn dado (a travs del constructor
//      sobrecargado) o bien llamando al mtodo Allocate de la clase CHeap.
//
// Modificaciones:
//     JM. 01/11/2002. Versin inicial para el artculo "Los montones en Win32" publicado
//                     en http://users.servicios.retecal.es/sapivi/prog/cpp/montones.html
//
//~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~
class HEAP_API CHeapBlock
{
private:
    CHeap     *m_heap;
    DWORD     m_size;
    LPVOID    m_memory;

public:
    CHeapBlock( 
        IN       CHeap  *a_heap, 
        IN CONST DWORD  a_size );

    virtual ~CHeapBlock( VOID );

    VOID SetSize( 
        IN CONST DWORD size );

    INT GetSize( VOID );

    LPVOID GetMemory( VOID );
};



class CHeapBlockList;



//~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~
//
// Clase: CHeap
//
// Descripcin: representa un montn dentro de Win32.
//
// Uso: al crear un objeto de este tipo, se crea un montn dedicado a travs de la funcin
//      HeapCreate. Para almacenar datos en l, se debe llamar a la funcin Allocate, la
//      cual devuelve un objeto CHeapBlock (que representa el bloque de memoria reservada).
//      Se pueden almacenar datos en el bloque a travs del mtodo CHeapBlock.GetMemory()
//
// Modificaciones:
//     JM. 01/11/2002. Versin inicial para el artculo "Los montones en Win32" publicado
//                     en http://users.servicios.retecal.es/sapivi/prog/cpp/montones.html
//
//~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~
class HEAP_API CHeap
{
private:
    HANDLE          m_handle;
    CHeapBlockList  *m_blocks;
    DWORD           m_serialize;
    BOOL            m_autoExtend;

public:
    // crea un nuevo montn indicando sus datos
    CHeap( 
        IN OPTIONAL CONST DWORD dwInitialSize = INITIAL_SIZE,
        IN OPTIONAL CONST DWORD flOptions     = HEAP_NO_SERIALIZE,
        IN OPTIONAL CONST DWORD dwMaximumSize = 0 );

    virtual ~CHeap( VOID );

    CHeapBlock *GetBlock(
        IN CONST INT i );

    CHeapBlockList *GetBlocks( VOID );

    CHeapBlock *Allocate( 
        IN CONST DWORD size );

    DWORD Compact( VOID );

    BOOL GetSerialize( VOID );

    VOID SetSerialize(
        IN CONST BOOL value );

    HANDLE GetHandle( VOID );

    BOOL GetAutoExtend( VOID );
};



//~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~
//
// Clase: CHeapList
//
// Descripcin: representa una lista de punteros (igual que la clase CList), que almacena
//              sus datos en un montn dedicado.
//
// Uso: el mismo que la clase TList: creando el objeto a travs de un constructor (indicando
//      el montn a utilizar o bien las caractersticas del montn a crear), y aadiendo
//      elementos a travs de los mtodos Add(Pointer) e Insert(Pointer).
//      Si los elementos apuntan a registros creados en tiempo de ejecucin, es recomendable
//      que la memoria de estos sea reservada dentro del mismo montn, a travs del mtodo
//      CHeapList.GetHeap.Allocate(tamao)
//
// Modificaciones:
//     JM. 01/11/2002. Versin inicial para el artculo "Los montones en Win32" publicado
//                     en http://users.servicios.retecal.es/sapivi/prog/cpp/montones.html
//
//~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~
class HEAP_API CHeapList
{
private:
    CHeap        *m_heap;    // el montn que se utiliza para almacenar los elementos
    CHeapBlock   *m_block;   // el bloque de memoria dentro del montn, donde reside FList

    BOOL    m_destroyHeap;   // indica si debe destruirse el montn

    LPVOID    *m_list;       // el puntero al array de punteros
    INT        m_count;      // el n actual de elementos
    INT        m_capacity;   // del n de elementos para el que se ha reservado memoria

protected:
    // aplica el algoritmo de crecimiento
    virtual VOID Grow( VOID );

public:
    // constructor indicando los datos del nuevo montn
    CHeapList( 
        IN OPTIONAL CONST DWORD dwInitialSize = INITIAL_SIZE,
        IN OPTIONAL CONST DWORD flOptions     = HEAP_NO_SERIALIZE,
        IN OPTIONAL CONST DWORD dwMaximumSize = 0 );
    // constructor indicando el montn a utilizar
    CHeapList(
        IN CHeap *a_heap );

    virtual ~CHeapList( VOID );

    // aadir elementos
    INT Add(
        IN CONST LPVOID item );

    VOID Insert(
        IN CONST INT    index, 
        IN CONST LPVOID item );

    // eliminar elementos
    virtual VOID Clear( VOID );
    VOID Delete( 
        IN CONST INT index );

    LPVOID GetItem(
        IN CONST INT index );
    
    VOID SetItem(
        IN CONST INT    index, 
        IN CONST LPVOID item );

    // buscar elementos
    INT GetIndex( 
        IN CONST LPVOID item );

    LPVOID *GetList( VOID );

    CHeap  *GetHeap( VOID );

    // reserva nueva memoria para los elementos
    VOID SetCapacity( 
        IN CONST INT NewCapacity );

    INT     GetCapacity( VOID );

    // establece el nuevo n de elementos, y hace crecer la lista si es necesario
    VOID SetCount( 
        IN CONST INT NewCount );

    INT  GetCount( VOID );
};



//~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~
//
// Clase: CHeapBlockList
//
// Descripcin: representa una lista de objetos de tipo CHeapBlock. Adems, esta lista
//              hereda de CHeapList, por lo que los datos los almacena en un montn dinmico
//              (creado por la propia CHeapList o bien pasado en el constructor.
//
// Uso: igual que la clase CHeapList, excepto que los objetos que maneja son de tipo
//      CHeapBlock en vez de LPVOID.
//
// Modificaciones:
//     JM. 01/11/2002. Versin inicial para el artculo "Los montones en Win32" publicado
//                     en http://users.servicios.retecal.es/sapivi/prog/cpp/montones.html
//
//~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~
class HEAP_API CHeapBlockList: public CHeapList
{
public:
    CHeapBlockList( 
        IN CHeap *a_heap );

    CHeapBlock *GetBlock( 
        IN CONST INT index );

    SetBlock( 
        IN CONST INT        index, 
        IN       CHeapBlock *item );
};




